package sun.awt.geom;

import java.awt.geom.Rectangle2D;

final class Order1 extends Curve
{
    private double x0;
    private double y0;
    private double x1;
    private double y1;
    private double xmin;
    private double xmax;
    
    public Order1(double xmin, double y0, double xmax, double y2, int n) {
        super(n);
        this.x0 = xmin;
        this.y0 = y0;
        this.x1 = xmax;
        this.y1 = y2;
        if (xmin < xmax) {
            this.xmin = xmin;
            this.xmax = xmax;
        }
        else {
            this.xmin = xmax;
            this.xmax = xmin;
        }
    }
    
    @Override
    public int getOrder() {
        return 1;
    }
    
    @Override
    public double getXTop() {
        return this.x0;
    }
    
    @Override
    public double getYTop() {
        return this.y0;
    }
    
    @Override
    public double getXBot() {
        return this.x1;
    }
    
    @Override
    public double getYBot() {
        return this.y1;
    }
    
    @Override
    public double getXMin() {
        return this.xmin;
    }
    
    @Override
    public double getXMax() {
        return this.xmax;
    }
    
    @Override
    public double getX0() {
        return (this.direction == 1) ? this.x0 : this.x1;
    }
    
    @Override
    public double getY0() {
        return (this.direction == 1) ? this.y0 : this.y1;
    }
    
    @Override
    public double getX1() {
        return (this.direction == -1) ? this.x0 : this.x1;
    }
    
    @Override
    public double getY1() {
        return (this.direction == -1) ? this.y0 : this.y1;
    }
    
    @Override
    public double XforY(double n) {
        if (this.x0 == this.x1 || n <= this.y0) {
            return this.x0;
        }
        if (n >= this.y1) {
            return this.x1;
        }
        return this.x0 + (n - this.y0) * (this.x1 - this.x0) / (this.y1 - this.y0);
    }
    
    @Override
    public double TforY(double n) {
        if (n <= this.y0) {
            return 0.0;
        }
        if (n >= this.y1) {
            return 1.0;
        }
        return (n - this.y0) / (this.y1 - this.y0);
    }
    
    @Override
    public double XforT(double n) {
        return this.x0 + n * (this.x1 - this.x0);
    }
    
    @Override
    public double YforT(double n) {
        return this.y0 + n * (this.y1 - this.y0);
    }
    
    @Override
    public double dXforT(double n, int n2) {
        switch (n2) {
            case 0: {
                return this.x0 + n * (this.x1 - this.x0);
            }
            case 1: {
                return this.x1 - this.x0;
            }
            default: {
                return 0.0;
            }
        }
    }
    
    @Override
    public double dYforT(double n, int n2) {
        switch (n2) {
            case 0: {
                return this.y0 + n * (this.y1 - this.y0);
            }
            case 1: {
                return this.y1 - this.y0;
            }
            default: {
                return 0.0;
            }
        }
    }
    
    @Override
    public double nextVertical(double n, double n2) {
        return n2;
    }
    
    @Override
    public boolean accumulateCrossings(Crossings crossings) {
        double xLo = crossings.getXLo();
        double n = crossings.getYLo();
        double xHi = crossings.getXHi();
        double n2 = crossings.getYHi();
        if (this.xmin >= xHi) {
            return false;
        }
        double n3;
        if (this.y0 < n) {
            if (this.y1 <= n) {
                return false;
            }
            n3 = this.XforY(n);
        }
        else {
            if (this.y0 >= n2) {
                return false;
            }
            n = this.y0;
            n3 = this.x0;
        }
        double n4;
        if (this.y1 > n2) {
            n4 = this.XforY(n2);
        }
        else {
            n2 = this.y1;
            n4 = this.x1;
        }
        if (n3 >= xHi && n4 >= xHi) {
            return false;
        }
        if (n3 > xLo || n4 > xLo) {
            return true;
        }
        crossings.record(n, n2, this.direction);
        return false;
    }
    
    @Override
    public void enlarge(Rectangle2D rectangle2D) {
        rectangle2D.add(this.x0, this.y0);
        rectangle2D.add(this.x1, this.y1);
    }
    
    @Override
    public Curve getSubCurve(double n, double n2, int n3) {
        if (n == this.y0 && n2 == this.y1) {
            return this.getWithDirection(n3);
        }
        if (this.x0 == this.x1) {
            return new Order1(this.x0, n, this.x1, n2, n3);
        }
        double n4 = this.x0 - this.x1;
        double n5 = this.y0 - this.y1;
        return new Order1(this.x0 + (n - this.y0) * n4 / n5, n, this.x0 + (n2 - this.y0) * n4 / n5, n2, n3);
    }
    
    @Override
    public Curve getReversedCurve() {
        return new Order1(this.x0, this.y0, this.x1, this.y1, -this.direction);
    }
    
    @Override
    public int compareTo(Curve curve, double[] array) {
        if (!(curve instanceof Order1)) {
            return super.compareTo(curve, array);
        }
        Order1 order1 = (Order1)curve;
        if (array[1] <= array[0]) {
            throw new InternalError("yrange already screwed up...");
        }
        array[1] = Math.min(Math.min(array[1], this.y1), order1.y1);
        if (array[1] <= array[0]) {
            throw new InternalError("backstepping from " + array[0] + " to " + array[1]);
        }
        if (this.xmax <= order1.xmin) {
            return (this.xmin == order1.xmax) ? 0 : -1;
        }
        if (this.xmin >= order1.xmax) {
            return 1;
        }
        double n = this.x1 - this.x0;
        double n2 = this.y1 - this.y0;
        double n3 = order1.x1 - order1.x0;
        double n4 = order1.y1 - order1.y0;
        double n5 = n3 * n2 - n * n4;
        double n6;
        if (n5 != 0.0) {
            double n7 = ((this.x0 - order1.x0) * n2 * n4 - this.y0 * n * n4 + order1.y0 * n3 * n2) / n5;
            if (n7 <= array[0]) {
                n6 = Math.min(this.y1, order1.y1);
            }
            else {
                if (n7 < array[1]) {
                    array[1] = n7;
                }
                n6 = Math.max(this.y0, order1.y0);
            }
        }
        else {
            n6 = Math.max(this.y0, order1.y0);
        }
        return Curve.orderof(this.XforY(n6), order1.XforY(n6));
    }
    
    @Override
    public int getSegment(double[] array) {
        if (this.direction == 1) {
            array[0] = this.x1;
            array[1] = this.y1;
        }
        else {
            array[0] = this.x0;
            array[1] = this.y0;
        }
        return 1;
    }
}
